import json
import os
import time

from nanicrypt.common import *


class ClientEncrypted:
    def __init__(self, data: dict, key: bytes, sign: str):
        self.data = data
        self.key = key
        self.sign = sign

    def __repr__(self):
        return "<ClientEncrypted "+str(self.key)+" "+self.sign+">"

    @property
    def json(self, *args, **kwargs):
        return json.dumps(self.data, *args, **kwargs)

    @property
    def sign_header(self):
        return {"sign": self.sign}

    @property
    def base64_key(self):
        return base64_encode(self.key)

class ClientNaniCrypt:
    def __init__(self, pubkey, sign_salt: str="<powered by NaniCrypt client-server encryption>"):
        if type(pubkey) is str:
            pubkey = pubkey.encode()
        self.pubkey = rsa.PublicKey.load_pkcs1(pubkey)
        self.salt = sign_salt

    def encrypt(self, data: dict, key: bytes=None) -> ClientEncrypted:
        if key is None:
            key = os.urandom(32)
        encrypted = {
            "m": base64_encode(aes_encrypt(json.dumps(data), key)),
            "k": base64_encode(rsa_encrypt(key, self.pubkey))
        }
        return ClientEncrypted(encrypted, key, self.make_sign(encrypted))

    def decrypt(self, data: dict, key: bytes) -> dict:
        a = aes_decrypt(
                base64_decode(data["m"]),
                key
            )
        return json.loads(
            a.decode()
        )

    def make_sign(self, payload: dict={}) -> str:
        return sha1(md5(self.salt) + sha1(json.dumps(payload))) + md5(
            str(int(round(time.time() / 100))))