import json
import os
import time

from nanicrypt.common import *


class ServerDecrypted:
    def __init__(self, data: dict, key: bytes, sign_valid: bool):
        self.data = data
        self.key = key
        self.sign_valid = sign_valid

    def __repr__(self):
        return "<ServerDecrypted " + str(self.key) + " " + ("<!> IE" if not self.sign_valid else "") + ">"

    @property
    def json(self, *args, **kwargs):
        return json.dumps(self.data, *args, **kwargs)

    @property
    def base64_key(self):
        return base64_encode(self.key)


class ServerNaniCrypt:
    def __init__(self, privkey, sign_salt: str = "<powered by NaniCrypt client-server encryption>"):
        if type(privkey) is str:
            privkey = privkey.encode()
        self.privkey = rsa.PrivateKey.load_pkcs1(privkey)
        self.salt = sign_salt

    def decrypt(self, data, sign: str="") -> ServerDecrypted:
        sign_valid = True
        if type(data) is bytes:
            data = data.decode()
        if type(data) is str:
            data = json.loads(data)
        if not self.check_sign(sign, data):
            sign_valid = False
        key = rsa_decrypt(base64_decode(data["k"]), self.privkey)
        msg = json.loads(aes_decrypt(base64_decode(data["m"]), key).decode())
        return ServerDecrypted(msg, key, sign_valid)

    def encrypt(self, data: dict, key: bytes) -> dict:
        encrypted = {
            "m": base64_encode(aes_encrypt(json.dumps(data), key)),
        }
        return encrypted

    def make_sign(self, payload: dict = {}) -> str:
        return sha1(md5(self.salt) + sha1(json.dumps(payload))) + md5(
            str(int(round(time.time() / 100))))

    def check_sign(self, sign: str, data: dict) -> bool:
        return sign == self.make_sign(data)


